@@ -85,7 +85,7 @@ struct _GnomeCanvasHacktextPriv {
 	double affine[6]; /* the text to world transform (NB! mirrored Y) */
 };
 
-GtkType
+GType
 gnome_canvas_hacktext_get_type (void)
 {
 	static GType type = 0;
@@ -262,21 +262,6 @@ get_bounds (GnomeCanvasHacktext *hacktext, double *bx1, double *by1, double *bx2
 	*by2 = bbox.y1;
 }
 
-#ifdef IFED_OUT_BY_CHEMA_TO_KILL_COMPILE_WARNING
-/* Convenience function to set a GC's foreground color to the specified pixel value */
-static void
-set_gc_foreground (GdkGC *gc, gulong pixel)
-{
-	GdkColor c;
-
-	if (!gc)
-		return;
-
-	c.pixel = pixel;
-	gdk_gc_set_foreground (gc, &c);
-}
-#endif
-
 static void
 gnome_canvas_hacktext_set_property (GObject               *object,
 				    guint                 param_id,
