@@ -143,7 +143,7 @@ gnome_print_dialog_init (GnomePrintDialog *gpd)
 	/* Empty */
 }
 
-void 
+static void 
 gnome_print_dialog_response_cb (GtkDialog *dialog,
 				gint response_id,
 				GnomePrintDialog *gpd)
@@ -198,6 +198,7 @@ gpd_create_job_page (GnomePrintDialog *gpd)
 	l = gtk_label_new ("");
 	text = g_strdup_printf ("<b>%s</b>", _("Print Range"));
 	gtk_label_set_markup (GTK_LABEL (l), text);
+	g_object_set_data (G_OBJECT (f), "label", l);
 	g_free (text);
 	gtk_frame_set_label_widget (GTK_FRAME (f), l);
 	gtk_widget_show (l);
@@ -372,7 +373,7 @@ gnome_print_dialog_construct (GnomePrintDialog *gpd, const guchar *title, gint f
 		l = gtk_label_new_with_mnemonic (_("Printer"));
 		gtk_widget_show (l);
 		
-		gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), hb, l);
+		gtk_notebook_prepend_page (GTK_NOTEBOOK (gpd->notebook), hb, l);
 		
 		/* Add a paper page */
 		p = gnome_paper_selector_new (gpd->config);
@@ -490,7 +491,7 @@ void
 gnome_print_dialog_construct_range_any (GnomePrintDialog *gpd, gint flags, GtkWidget *range_widget,
 					const guchar *currentlabel, const guchar *rangelabel)
 {
-	GtkWidget *f, *r;
+	GtkWidget *f, *r, *l, *b;
 
 	g_return_if_fail (gpd != NULL);
 	g_return_if_fail (GNOME_IS_PRINT_DIALOG (gpd));
@@ -511,6 +512,20 @@ gnome_print_dialog_construct_range_any (GnomePrintDialog *gpd, gint flags, GtkWi
 		gtk_widget_show (r);
 		gtk_widget_show (gpd->job);
 		gtk_container_add (GTK_CONTAINER (f), r);
+
+		l = g_object_get_data (G_OBJECT (f), "label");
+		b = g_object_get_data (G_OBJECT (r), "current");
+		if (b !=NULL)
+			gnome_print_set_atk_relation (l, GTK_WIDGET (b));
+		b = g_object_get_data (G_OBJECT (r), "all");
+		if (b !=NULL)
+			gnome_print_set_atk_relation (l, GTK_WIDGET (b));
+		b = g_object_get_data (G_OBJECT (r), "range");
+		if (b !=NULL)
+			gnome_print_set_atk_relation (l, GTK_WIDGET (b));
+		b = g_object_get_data (G_OBJECT (r), "selection");
+		if (b !=NULL)
+			gnome_print_set_atk_relation (l, GTK_WIDGET (b));
 	}
 
 	g_object_set_data (G_OBJECT (f), "range", r);
@@ -538,9 +553,6 @@ gnome_print_dialog_construct_range_page (GnomePrintDialog *gpd, gint flags, gint
 	if (flags & GNOME_PRINT_RANGE_RANGE) {
 		GtkWidget *l, *sb;
 		GtkObject *a;
-		AtkRelationSet *relation_set;
-		AtkRelation *relation;
-		AtkObject *relation_targets[1];
 		AtkObject *atko;
 
 		hbox = gtk_hbox_new (FALSE, 3);
@@ -764,3 +776,43 @@ gnome_print_dialog_run (GnomePrintDialog const *gpd)
 	return response;
 }
 
+/**
+ * gnome_print_set_atk_relation:
+ *
+ * Sets Atk Relation
+ *
+ **/
+void
+gnome_print_set_atk_relation (GtkWidget *label, GtkWidget *widget)
+{
+	AtkRelationSet *relation_set;
+	AtkRelation *relation;
+	AtkObject *relation_targets[1];
+	AtkObject *atk_widget, *atk_label;
+
+	atk_label = gtk_widget_get_accessible (label);
+	atk_widget = gtk_widget_get_accessible (widget);
+
+	/* Add a LABEL_FOR relation from the label to the label_for
+	widget. */
+
+	relation_set = atk_object_ref_relation_set (atk_label);
+	relation_targets[0] = atk_widget;
+	relation = atk_relation_new (relation_targets, 1,
+				     ATK_RELATION_LABEL_FOR);
+	atk_relation_set_add (relation_set, relation);
+	g_object_unref (G_OBJECT (relation));
+	g_object_unref (G_OBJECT (relation_set));
+
+	/* Add a LABELLED_BY relation from the mnemonic widget to the
+	label. */
+
+	relation_set = atk_object_ref_relation_set (atk_widget);
+	relation_targets[0] = atk_label;
+	relation = atk_relation_new (relation_targets, 1,
+				     ATK_RELATION_LABELLED_BY);
+	atk_relation_set_add (relation_set, relation);
+	g_object_unref (G_OBJECT (relation));
+	g_object_unref (G_OBJECT (relation_set));
+}
+
