@@ -33,6 +33,17 @@ libgnomeprintui_gettext (const char *msgid)
 	static gboolean initialized = FALSE;
 
 	if (!initialized) {
+#ifdef G_OS_WIN32
+		gchar *localedir = g_win32_get_package_installation_subdirectory (NULL,
+										  "libgnomeprintui-2-2-0.dll",
+										  "/share/locale");
+
+		if (localedir) {
+			bindtextdomain (GETTEXT_PACKAGE, localedir);
+			g_free (localedir);
+		}
+		else
+#endif
 		bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 		bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 		initialized = TRUE;
